import time
from microbit import i2c

trameCmdDebut = "55AA11"


class HuskyLens:
    def __init__(self):
        self.adresse_i2c = 0x32
        self.derniereCmdEnvoyee = ""
        
    def ecritureI2C(self, cmd):
        self.derniereCmdEnvoyee = cmd
        i2c.write(self.adresse_i2c, cmd)

    def calculChecksum(self, hexStr):
        total = 0
        for i in range(0, len(hexStr), 2):
            total += int(hexStr[i:i+2], 16)
        hexStr = hex(total)[-2:]
        return hexStr
        
   
    def convChaineSeqOctet(self, cmd):
        byte_data = []
        for i in range(0, len(cmd), 2):
            byte_data.append(int(cmd[i:i+2], 16))
        return bytes(byte_data)

    def splitCommandToParts(self, str):
        headers = str[0:4]
        adresse_i2c = str[4:6]
        data_length = int(str[6:8], 16)
        command = str[8:10]
        if(data_length > 0):
            data = str[10:10+data_length*2]
        else:
            data = []
        checkSum = str[2*(6+data_length-1):2*(6+data_length-1)+2]
        return [headers, adresse_i2c, data_length, command, data, checkSum]

    def getBlockOrArrowCommand(self):
        byteString  =i2c.read(self.adresse_i2c,5)
        byteString +=i2c.read(self.adresse_i2c,byteString[3]+1)
        commandSplit = self.splitCommandToParts(''.join(['%02x' % b for b in byteString]))
        return commandSplit[4]

    def lectureDonnees(self):
        try:
            byteString = i2c.read(self.adresse_i2c,5)
            byteString += i2c.read(self.adresse_i2c,byteString[3]+1)
            commandSplit = self.splitCommandToParts(''.join(['%02x' % b for b in byteString]))
            if(commandSplit[3] == "2e"):
                return "Knock Recieved"
            else:
                returnData = []
                numberOfBlocksOrArrow = int(
                    commandSplit[4][2:4] + commandSplit[4][0:2], 16)
                numberOfIDLearned = int(
                    commandSplit[4][6:8] + commandSplit[4][4:6], 16)
                frameNumber = int(
                    commandSplit[4][10:12] + commandSplit[4][8:10], 16)
                for i in range(numberOfBlocksOrArrow):
                    returnData.append(self.getBlockOrArrowCommand())
                finalData = []
                tmp = []
                for i in returnData:
                    tmp = []
                    for q in range(0,len(i),4):
                        tmp.append(int(i[q:q+2],16)+int(i[q+2:q+4],16))
                    finalData.append(tmp)
                    tmp = []
                return finalData
        except:
             #print("erreur lecture")
             return []
   
   
    def id_present(self, id_a_verifier):
        data_list = self.requete("cadres")     
        return any(element[4] == id_a_verifier for element in data_list)
    
        
    def requete(self, command_type=""):
        command_dict = {
            'test_connexion': '002c3c',
            'par_defaut': '002030',
            'cadres': '002131',
            'fleches': '002232',
            'appris': '002333',
            'cadres_appris': '002434',
            'fleches_apprises': '002535'
        }
        
        if command_type not in command_dict:
            raise ValueError("Commande" + command_type + " non reconnue.")
        cmd = self.convChaineSeqOctet(trameCmdDebut + command_dict[command_type])
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
    

    def mode_reconnaisance(self, mode):
        modes = {
            "suivi_ligne": "022d030042",
            "reconnaissance_visage": "022d00003f",
            "suivi_objet": "022d010040",
            "reconnaissance_objet": "022d020041",
            "reconnaissance_couleur": "022d040043",
            "reconnaissance_tag": "022d050044",
            "classification_objet": "022d060045"
        }
        
        if mode not in modes:
            raise ValueError("Mode " + mode + " inconnu.")
        
        cmd = self.convChaineSeqOctet(trameCmdDebut + modes[mode])
        self.ecritureI2C(cmd)
        return self.lectureDonnees()


    # tous les cadres et flèches par ID donné
    def command_request_by_id(self, idVal):
        idVal = "{:04x}".format(idVal)
        idVal = idVal[2:]+idVal[0:2]
        cmd = trameCmdDebut+"0226"+idVal
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
    
    #tous les cadres par ID donné
    def command_request_blocks_by_id(self, idVal):
        idVal = "{:04x}".format(idVal)
        idVal = idVal[2:]+idVal[0:2]
        cmd = trameCmdDebut+"0227"+idVal
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
    
    #toutes les flèches par ID donné
    def command_request_arrows_by_id(self, idVal):
        idVal = "{:04x}".format(idVal)
        idVal = idVal[2:]+idVal[0:2]
        cmd = trameCmdDebut+"0228"+idVal
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
        
    def oublier_textes_id(self):
        cmd = trameCmdDebut
        dataLength = 0
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x35)
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
    
    def prendre_photo_sd(self):
        cmd = trameCmdDebut
        dataLength = 0
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x30)
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
    
    def oublier_id_appris(self):
        cmd = trameCmdDebut
        dataLength = 0
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x37)       
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
  
    def capture_ecran_sd(self):
        cmd = trameCmdDebut
        dataLength = 0
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x39)      
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
  
  
  
    def texte_perso(self, text, x, y):
        textLength = len(text)
        dataLength = textLength+4
        cmd = trameCmdDebut
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x34) 
        cmd += "{:02x}".format(dataLength)  
        if x > 255:
            data_1 = 0xff
            cmd += "{:02x}".format(data_1)      
            data_2 = x % 256        
            cmd += "{:02x}".format(data_2)   
        else:
            cmd += "{:02x}".format(0)
            cmd += "{:02x}".format(x)
        cmd += "{:02x}".format(y)
        
        for char in text:
            cmd += "{:02x}".format(ord(char))
            
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()  

    def apprendre(self, id):
        cmd = (trameCmdDebut + "02" + "36")
        cmd += "{:02x}".format(id & 0xff)
        cmd += "{:02x}".format((id >> 8) & 0xff)          
        cmd += self.calculChecksum(cmd)        
        self.ecritureI2C(self.convChaineSeqOctet(cmd))       
        return self.lectureDonnees()


    def donner_texte_id(self, id, name):
        nameLength = len(name)
        dataLength = nameLength+3
        cmd = trameCmdDebut        
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x2f)   
        
        cmd += "{:02x}".format(id)
        cmd += "{:02x}".format(nameLength+1)
        for char in name:
            cmd += "{:02x}".format(ord(char))           
        cmd += "{:02x}".format(0)
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
     
    def enregistrer_modele_sd(self,index):
        cmd = trameCmdDebut        
        dataLength = 2
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x32)   
        index = [index & 0xff, (index >> 8) & 0xff]   
        cmd += "{:02x}".format(index[0])
        cmd += "{:02x}".format(index[1])        
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()
  
    def charger_modele_sd(self, index):
        cmd = trameCmdDebut        
        dataLength = 2
        cmd += "{:02x}".format(dataLength)
        cmd += "{:02x}".format(0x33)   
        index = [index & 0xff, (index >> 8) & 0xff]   
        cmd += "{:02x}".format(index[0])
        cmd += "{:02x}".format(index[1])        
        cmd += self.calculChecksum(cmd)
        cmd = self.convChaineSeqOctet(cmd)
        self.ecritureI2C(cmd)
        return self.lectureDonnees()